(* $Id: hydro_client_connector.mli 15333 2007-11-23 16:41:36Z gerd $ *)

(** Connect a client with a remote endpoint *)

open Hydro_types


type socket =
    [ `Connect_addr of Unix.sockaddr * transport_protocol_type
	(* The socket will be connected to this address *)
    | `Connect_inet of string * int * transport_protocol_type
	(* (host, port, type) *)
    | `Unconnected_datagram of Unix.socket_domain
        (* The socket is an unconnected datagram socket *)
    | `Descriptor of unit -> descriptor
        (* The socket is given by this descriptor *)
    ]
    (** How to get a socket *)


type connection =
    < ctrl : Hydro_transport.hydro_multiplex_controller;
      shutdown : unit -> unit
    >

class type mplex_config =
object
  method multiplexing : 
    descriptor -> Unixqueue.event_system -> connection Uq_engines.engine
end

val default_mplex_config : mplex_config
class default_mplex_config : mplex_config
  (* The default mplex_config works for simple stream and datagram sockets *)

type connector =
    [ `Mplex_controller of Hydro_transport.hydro_multiplex_controller
    | `Mplex_socket of socket * mplex_config
    | `Proxy_endpoint of endpoint * network_port option
    ]
   (* Various ways of connecting to a server *)
   (* `Proxy_endpoint(ep,port): [port] is the DNS-resolved port of the
      port mentioned in [ep]
    *)

val connect_engine : connector -> 
                     Unixqueue.event_system ->
                     connection Uq_engines.engine
  (** An engine that is able to create and, if requested, to connect the socket
   *)

val register_endpoint_type : endpoint_type ->
                             (endpoint -> network_port option -> socket * mplex_config) ->
                             proxy_mode list ->
                             unit
  (** By default, this module knows only about `TCP and `UDP endpoint types.
      It is possible to register further types.
   *)

val proxy_supports_endpoint_type : endpoint_type -> proxy_mode -> bool
  (** Whether a handler is registered for an endpoint type, and the
      proxy mode is possible
   *)
