(* $Id: hydro_endpoint.mli 15333 2007-11-23 16:41:36Z gerd $ *)

open Hydro_types

(** Endpoints (clients and servers) *)


module Client : sig
  (** This module encapsulates the client interface *)

  type t
    (** A live client *)

  val create : system -> 
               Hydro_client_connector.connector -> 
               Unixqueue.event_system -> 
                 t
    (** Create a new client *)

  val configure : t -> client_params -> unit
   (** Configures the client. There is a default configuration for freshly
      created clients (XXX).
    *)

  type response =
      < condition : [ `Success | client_condition ];
        out_args : value array;
	result : value;
           (* Raises [Client_condition] if [condition] is not [`Success] *)
	peer : Unix.sockaddr option;
	mode : op_mode;   (* Repeats the info whether idempotent *)
	client : t option;
	>
    (** A friendly version of response messages *)

  val twoway_call : 
        ?facet:string ->
         t -> identity -> hintf -> string -> call_params ->
         value array ->
         (response -> unit) ->
           unit
   (** Starts a twoway call *)

(* TODO:
   add_batch_call
 *)

  val queued_requests : t -> int
    (** Number of requests in the queue (that are not yet responded) *)

  val is_up : t -> bool
    (** Whether the client is up (accepts new requests) *)

  val is_idle : t -> bool
    (** Whether the client is up and idle *)

  val client_id : t -> int
    (** Returns an identifier for the client *)

  val shutdown : t -> unit
    (** Shut the client gracefully down (by sending a close connection message) *)

  val shutdown_when_idle : t -> unit
    (** Shutdown gracefully the next time the client becomes idle *)

  val abort : t -> unit
    (** Closes the connection immediately (no close connection message). *)

  val graceful : t -> bool
    (** The graceful flag is initially true, but set to false when a fatal
        error or a non-graceful shutdown happens
     *)

  val pool_connect : t -> (unit -> unit) -> unit
    (** Used by [Hydro_proxy.pool]: The callback function is invoked when
        [is_up] becomes false
     *)
end
