(* $Id: hydro_lm_IceObject.ml 15330 2007-11-23 14:27:43Z gerd $ *)

open Hydro_types
open Hydro_lm

type proxy_name = [ `Ice_Object ]
and t_Ice_Object = pr_Ice_Object option
and pr_Ice_Object = [ `Ice_Object ] Hydro_lm.proxy_reference

let ofpr_Ice_Object = Hydro_lm.Unsafe.of_proxy_reference
let topr_Ice_Object = Hydro_lm.Unsafe.to_proxy_reference

class type r_Ice_Object_ice_ping =
object
  method hydro_response : Hydro_lm.client_response
  method result : unit
end

let to_r_Ice_Object_ice_ping (result : Hydro_lm.client_response) =
object
  method hydro_response = result
  method result =
    match result#result with
      | `Nothing -> ()
      | _ -> assert false
end

class type poi_Ice_Object =
object
  method ice_ping : unit -> r_Ice_Object_ice_ping Hydro_lm.call_suspension_t
end


class type po_Ice_Object =
object
  inherit Hydro_proxy.proxy_t
  inherit poi_Ice_Object
  method hydro_proxy_reference : pr_Ice_Object
end


class pci_Ice_Object (proxy : Hydro_proxy.proxy_t) =
  let intf =
    try Hashtbl.find proxy#hydro_env#system#interfaces "::Ice::Object"
    with Not_found -> assert false in
object(self)
  method ice_ping() =
    new Hydro_lm.call_suspension
      (proxy # hydro_twoway_call
           intf
           "ice_ping"
           [| |]
      )
      to_r_Ice_Object_ice_ping
      proxy#hydro_env#event_system
end


class pc_Ice_Object proxy_env pr =
  let proxy = 
    Hydro_proxy.proxy
      ~env:proxy_env
      ~addr:(Hydro_lm.Unsafe.unwrap_proxy (pr : pr_Ice_Object) 
	       : Hydro_types.proxy_addr :> Hydro_proxy.extended_proxy_addr)
      () in
object(self)
  inherit pci_Ice_Object proxy
  inherit Hydro_proxy.proxy_delegation proxy
  method hydro_proxy_reference = pr
end


let pc_Ice_Object = new pc_Ice_Object


let unchecked_pr_Ice_Object pr = 
  Hydro_lm.Unsafe.wrap_proxy (Hydro_lm.Unsafe.unwrap_proxy pr)


class type od_Ice_Object =
object
  method facets : unit ref
end

class type o_Ice_Object =
object
  inherit od_Ice_Object
  inherit Hydro_lm.object_base
end

type or_Ice_Object = object_base

exception O_Ice_Object of o_Ice_Object

let as_Ice_Object ov =
  try 
    let _ = 
      (ov : #object_base :> object_value) # hydro_inflate "::Ice::Object" in
    assert false
  with
    | O_Ice_Object ov' -> ov'
    | _ -> raise Hydro_lm.Invalid_coercion

let wrap_Ice_Object (o : o_Ice_Object) = (o :> object_base)
let unwrap_Ice_Object o = as_Ice_Object o

let ofor_Ice_Object = Hydro_lm.value_of_object

let toor_Ice_Object (v : value) =
  match v with
    | `Class vr ->
	Hydro_lm.object_of_class_repr vr
    | _ ->
	raise(Unmarshal_error("Hydro_lm_IceObject.toor_Ice_Object"))


class delegate_od_Ice_Object od : od_Ice_Object =
object
  method facets = od#facets
end


let dec_Ice_Object slices =
  (* let parent_vals, slices = dec_parent slices in *)
  match slices with
    | (`Decoded("::Ice::Object", [| facets |] )) :: slices' ->
	( () (* ,parent_vals *), slices')
    | _ ->
	raise(Unmarshal_error("Class value is not an ::Ice::Object"))
	

class mk_od_Ice_Object (facets : unit) =
object
  val facets = ref facets
  method facets = facets
end


(* A subclass C of Ice::Object would do:

  let dec_C slices =
    let parent_vals, slices' = dec_Ice_Object slices in
    match slices' with
      | (`Decoded("::C", [| foo; ... |] )) :: slices'' ->
	  ( ( of_foo foo, ... ,parent_vals), slices'')
      | _ ->
	  raise(Unmarshal_error("Class value is not a ::C"))

  class mk_od_C (foo, ..., parent_vals) =
  object
    inherit mk_od_Ice_Object parent_vals
    method foo = ref foo
  end
 *)

let enc_Ice_Object (od : #od_Ice_Object) =
  let slice = `Decoded("::Ice::Object",
		       [| (* of_XXX ! (od # facets) *)
			  `Sequence [| |] 
		       |]) in
  [ slice ]


class sliced_od_Ice_Object od : sliced_base =
object
  method hydro_slices = List.rev (enc_Ice_Object od)
  method hydro_effective_id = "::Ice::Object"
end

(* A subclass C of Ice::Object would do:

   let enc_C od =
     let slice = `Decoded("::C",
                          [| of_foo_type (! ( od#foo ) ) |] ) in
     slice :: enc_Ice_Object od

   class slice_od_C od =
   object
     method hydro_slices = List.rev (enc_C od)
     method hydro_effective_id = "::C"
   end
 *)


class mk_Ice_Object od =
object(self)
  inherit delegate_od_Ice_Object od
  inherit sliced_od_Ice_Object od
  method hydro_inflate id =
    match id with
      | "::Ice::Object" ->
	  raise(O_Ice_Object(self : #o_Ice_Object :> o_Ice_Object))
      | _ ->
	  raise Invalid_coercion
end


class restore_Ice_Object (sv :sliced_value) =
  let od = new mk_od_Ice_Object (fst (dec_Ice_Object sv#hydro_slices)) in
  mk_Ice_Object od


let ctor_Ice_Object sv =
  (new restore_Ice_Object sv :> object_base)



let defterm_Ice_Object =
  lazy
    ( object
	method name = "::Ice::Object"
	method super = []
	method elements =
	  [ ( object
		method name = "ice_ping"
		method mode = `Normal
		method in_args = [| |]
		method in_classes = false
		method out_args = [| |]
		method result = `Void
		method out_classes = false
              end : Hydro_types.hfunction
	    )
	  ]
      end : Hydro_types.hintf
    ) 


let class_defterm_Ice_Object =
  lazy
    ( object
	method name = "::Ice::Object"
	method super = None
	method elements = [| "facets", `Sequence `Void |]
      end : Hydro_types.hclass
    )


let fill_system sys =
  let intf = Lazy.force defterm_Ice_Object in
  Hashtbl.add 
    sys#interfaces
    intf#name
    intf;
  let cls = Lazy.force class_defterm_Ice_Object in
  Hashtbl.add
    sys#classes
    cls#name
    cls;
  Hashtbl.add
    sys#ctors
    cls#name
    ctor_Ice_Object
;;

